// Technische Richtlinie FA205 
// Bibliothek:       lcd.h
// Controller:       AT89C5131
// Version:          1.0
// erstellt am:      30.9.2015
// letzte nderung:  12.10.2015
// Autor:            Rahm

#ifndef _LCD_H_
#define _LCD_H_

#include "controller.h"
//**** Schalter ********************************************************
#define LCD_LEN 16              // Display-Zeilenlnge 20 oder 16 (default)
#define LCD_MINIBOARD           // Displayschnittstelle fr Miniboard GewTechnSchuleOffenburg Neumaier
//#define LCD_STACKTOLEARN        // Displayschnittstelle fr StackToLearn-Board
//#define LCD_EXTENSION         // schaltet Zusatzfunktionen frei
//**********************************************************************

//Anzeige Einstellungen
#define CURSOR      0x0c        // 0b00001100  ->  Display on, Cursor off, Blink off
#define DISPLAY     0x14        // 0b00010100  ->  Cursor shift right

#define ZEILE1      0x00        // Startadresse fr erste Displayzeile
#define ZEILE2      0x40

#if (LCD_LEN == 16)
 #define ZEILE3     0x10        // Fr 16 Zeichen LCDs Zeile3 = 0x10
 #define ZEILE4     0x50        // "       "       "   Zeile4 = 0x50
#elif (LCD_LEN == 20)
 #define ZEILE3     0x14        // Fr 20 Zeichen LCDs Zeile3 = 0x14
 #define ZEILE4     0x54        // "       "       "   Zeile4 = 0x54
#endif

// Spezielle Definitionen fr den 4 Bit Betrieb
#ifdef LCD_MINIBOARD            // Miniboard (Bubbers)
 #define LCD_PORT P0 
                                // P0 .7 .6 .5 .4 .3 .2 .1 .0
                                //    RS RW -- EN D3 D2 D1 D0
 #define RS 0x07
 #define RW 0x06
 #define EN 0x04
 //#define LCD_BUSY     3         // An Port 0 kann das Busy-Flag (D3) nicht abgefragt werden
#endif
#ifdef LCD_STACKTOLEARN         // StackToLearn (Schabelski)	
 #define LCD_PORT P2
                                // P2 .7 .6 .5 .4 .3 .2 .1 .0
                                //    -- RS RW EN D3 D2 D1 D0 														
 #define RS 0x06
 #define RW 0x05
 #define EN 0x04
 #define LCD_BUSY 3             // An Port 2 kann das Busy-Flag (D3) abgefragt werden
#endif


//Funktionsdefinitionen fr Technische Richtlinie FA205
extern void lcd_init      ( void );                            // Initialisierung des Displays
extern void lcd_clear     ( void );
extern void lcd_setcursor ( uint8_t row, uint8_t column );     // Setzen der Cursorposition
                                                               // row (Zeile) = 1,2, ...
                                                               // column (Spalte) = 1,2,3, ...
extern void lcd_print     ( uint8_t  text[] );                 // Textausgabe ab Cursorposition, bis '\0'
extern void lcd_char      ( uint8_t  value  );                 // Ausgabe eines Zeichens an der aktuellen Cursorposition
extern void lcd_byte      ( uint8_t  value  );                 // 0 ... 255
extern void lcd_int       ( uint16_t value  );                 // 0 ... 65535

//erweiterte Funktionen nicht Teil der Technischen Richtlinie FA205
#ifdef LCD_EXTENSION
 extern void lcd_defchar   ( uint8_t *pixtab, uint8_t charNr );
 // Definition von 7(8) eigenen Zeichen charNr: (0)1 bis 7
 // charNr = 0 funnktioniert nur, wenn das Zeichen nicht in Zeichenketten ('\0' = 0) verwendet wird.
 // Falsch :  lcd_print("Mein \0 Zeichen");     // " Zeichen" wird nicht mehr angezeigt!
 // Richtig:  lcd_print("Mein ");  lcd_char(0); lcd_print(" Zeichen");
 // Richtig:  lcd_print("Mein \2 Zeichen");     // \1...\7  = selbstdefinierte Zeichen 1...7
 
 extern void lcd_clearline ( uint8_t lineNr );
 #endif
#endif
