// Technische Richtlinie FA205 
// Bibliothek:       in_out.c
// Controller:       8051 (at89c5131)
// Version:          1.0
// erstellt am:      30.9.2015
// letzte nderung:  02.10.2015
// Autor:            Rahm	

#include "in_out.h"

// Definition der Funktionen
//***************************************************************
// ab hier Funktionen fr Port-Ein-/Ausgabe
//***************************************************************
void bit_init(uint8_t port, uint8_t bitnr, uint8_t direction)
{
  uint8_t temp;
	
  temp = byte_read(port);
	
  if (direction == OUT)
  {
    temp &= ~(0x01 << bitnr);     // Ausgang auf 0 setzen
  }
  else
  {
    temp |= (0x01 << bitnr);      // Input: internes Port-Flipflop gesetzt.
  }	
  
  byte_write(port,temp);        // Port schreiben
  
  #ifdef NOPULLUPS
  AUXR |= (0x01 << 7);	// Atmel: Pull-Ups werden beim Schreiben von 0 in den Port deaktiviert. (Gilt generell fr die Ports 1..3)
  #endif
}

void byte_init(uint8_t port, uint8_t direction)
{	
  if (direction == OUT)
  {
    byte_write(port,0x00);      // Ausgnge auf 0 setzen
  }
  else
  {
    byte_write(port,0xff);      // Als Input werden die internen Port-Flipflop auf 1 gesetzt.
  }
	
  #ifdef NOPULLUPS
  AUXR |= (0x01<<7);            // Atmel: Pull-Ups werden beim Schreiben von 0 in den Port deaktiviert. (Gilt generell fr die Ports 1..3)
  #endif
}

uint8_t bit_read(uint8_t port, uint8_t bitnr)
{
  uint8_t temp;
	
  temp = byte_read(port);
  return ((temp>>bitnr) & 0x01);
}

void bit_write(uint8_t port, uint8_t bitnr, uint8_t value)
{
  uint8_t temp;
	
  temp = byte_read(port);           // Port einlesen

  value &= 0x01;                    // value darf nur 0 oder 1 sein!
  if (value == 1)   temp |= (0x01<<bitnr);
  else              temp &= ~(0x01<<bitnr);
	
  byte_write(port,temp);            // Port wieder ausgeben
}

uint8_t byte_read(uint8_t port)
{
  switch (port)
  {
    case 0: return P0; break;
    case 1: return P1; break;
    case 2: return P2; break;
    case 3: return P3; break;
  }
  return 0;
}

//PORT als Ausgabe
void byte_write(uint8_t port, uint8_t value)
{
  switch (port)
  {
    case 0: P0 = value; break;
    case 1: P1 = value; break;
    case 2: P2 = value; break;
    case 3: P3 = value; break;
  }
}

//***************************************************************
// ab hier PWM-Funktionen
//***************************************************************
#if   (PWM_PIN == 3)
  #define CCAPL CCAP0L
  #define CCAPH CCAP0H
  #define CCAPM CCAPM0
#elif (PWM_PIN == 4)
  #define CCAPL CCAP1L
  #define CCAPH CCAP1H
  #define CCAPM CCAPM1
#elif (PWM_PIN == 5)
  #define CCAPL CCAP2L
  #define CCAPH CCAP2H
  #define CCAPM CCAPM2
#elif (PWM_PIN == 6)
  #define CCAPL CCAP3L
  #define CCAPH CCAP3H
  #define CCAPM CCAPM3
#elif (PWM_PIN == 7)
  #define CCAPL CCAP4L
  #define CCAPH CCAP4H
  #define CCAPM CCAPM4
#endif

void pwm_init(void)
{
  CMOD = 0x00;                  // PWM-Frequenz ca. 3,9 kHz
  CCAPL = CCAPH = 0xff-0x7F;    // Tastgrad-Initialwert 50%	

  P1 &= ~(1<<PWM_PIN);          // PWM-Pin auf Low setzen
}

void pwm_start()
{
  CL = 0x00;                    // PCA Counter Lowbyte auf 0 setzen
  CR = 1;                       // PCA Counter starten
  CCAPM = 0x42;				          // 8 Bit PWM-Modus

  P1 |= (1<<PWM_PIN);           // P1.7 auf High setzen							
}

void pwm_stop()
{
  CCAPM = 0x00;                 // 8 Bit PWM-Modus
  P1 &= ~(1<<PWM_PIN);          // P1.7 auf Low setzen
  
  CR = 0;                       // PCA Counter stoppen
}

void pwm_duty_cycle ( uint8_t value )
{
  CCAPH = value;           // neuer Tastgrad
}

//***************************************************************
// ab hier ADC-Funktionen
//***************************************************************
//***** verwendete I2C-Adressen fr den adc081c021/027 ****************
#ifdef _ADC081C021_
#define ADR_READ    0xAB      // 10101011  ->  10101 fest, 01 durch Platine, 1 lesen		
#define ADR_WRITE   0xAA      // 10101010  ->  10101 fest, 01 durch Platine, 0 schreiben		
#endif
//***** verwendete I2C-Adressen und Kontrollbyte fr den PCF8591 ****************
#ifdef _PCF8591_
#define ADDR_R      0x91      // 10010001  ->  1001 fest, 000 durch Platine, 1 Wert lesen		
#define ADDR_W      0x90      // 10010000  ->  1001 fest, 000 durch Platine, 0 Wert schreiben		
#define CNTR_BYTE   0x40      // 01000000  ->  Kontrollbyte 4 einzelne ADU, Kanal= Offset dazu
#endif
//***** Konstanten
#define CH1 0x00    // Kanal 1: Addr 0
#define CH2 0x01    // Kanal 2: Addr 1

uint8_t adc_in(uint8_t ch);   // Prototyp! Lokale Funktion mit Kanalwahl

// Initialisierung des ADU. Ist hier eigentlich nicht ntig!
void adc_init(void)
{
  i2c_init();                 // I2C-Bus initialisieren	(Grundzustand)
}

// Einlesen des ADC-Kanals 1
uint8_t adc_in1( void )
{
  return (adc_in(CH1));
}

// Einlesen des ADC-Kanals 2
uint8_t adc_in2( void )
{
  return (adc_in(CH2));
}

#ifdef _ADC081C021_
// Funktion mit Kanal-Parameter fr ADC081C021. 
// Beim adc081c021/027 gibts nur einen Kanal! CH2 gibt immer 0 zurck!
uint8_t adc_in(uint8_t ch )
{
  uint8_t tempH,tempL;
  if (ch == CH1)
  {  i2c_init();                 // I2C-Bus initialisieren	(Grundzustand)						
 	
    // Anschlieend erfolgt das Lesen des Analogwertes
    i2c_start();                // Startbedingung
    i2c_write(ADR_READ);        // Adresse des IC und Lesewunsch

    tempH = i2c_read(ACK);      // Zuerst High-Byte lesen
    tempH <<= 4;                //   und 4 MSB linksbndig!

    tempL = i2c_read(NACK);     // dann Low-Byte lesen
    tempL >>= 4;                //   und 4 LSB rechtsbndig!

    i2c_stop();                 // Stoppbedingung

    return (tempH | tempL);     // High-und Low-Nibble zusammenfassen
  }
  else return (0);
}
#endif
#ifdef _PCF8591_
// Funktion mit Kanal-Parameter fr PCF8591
uint8_t adc_in(uint8_t ch)  
{
  uint8_t temp;	
	
  i2c_init();                 // I2C-Bus initialisieren	(Grundzustand)						
  // Zunchst muss das Controlbyte (Kanalwahl) geschrieben werden.
  i2c_start();                // Startbedingung
  i2c_write(ADDR_W);          // Adresse des IC und Schreibwunsch (zur Kanalwahl)		
  i2c_write((ch & 0x03) | CNTR_BYTE);     // Controll-Byte mit Kanalnummer
  i2c_stop();                 // Stoppbedingung
	
  // Anschlieend erfolgt das Lesen des Analogwertes
  i2c_start();                // Startbedingung
  i2c_write(ADDR_R);          // Adresse des IC und Lesewunsch
  i2c_read(ACK);              // Dummy Wandlung (erster Wert ist unbrauchbar!)
  temp = i2c_read(NACK);      // neuen Wert abholen
  i2c_stop();                 // Stoppbedingung
	
  return (temp);
}
#endif


