// Technische Richtlinie FA205 
// Bibliothek:       interrupt.c
// Controller:       alle 8051 (at89c5131)
//     Verwendet externen Interrupt 0 und Timer_2
// Version:          1.0
// erstellt am:      18.02.2015
// letzte nderung:  08.11.2015
// Autor:            Rahm

#include "interrupt.h"

#ifdef _EXT_INT0_
 #define IT_ 0            // Trigger-Einstellung   (TCON)
 #define IE_ 1            // Interruptanforderungsflag (TCON)
 #define EX_ 0            // Einzelfreigabe-Flag (IEN0)
#endif
#ifdef _EXT_INT1_
 #define IT_ 2
 #define IE_ 3
 #define EX_ 2
#endif

#define  EA_ 7

void (*my_interrupt) (void);      // Pointer auf Interrupt und Timer ISR
void (*my_timer)     (void);

void ext_interrupt_init ( void (*ip) (void) )
{
  my_interrupt = ip;
  TCON  |=  (0x01 << IT_);  //  IT0 = 1;     // auf fallende Flanke triggern
  TCON  &= ~(0x01 << IE_);  //  IE0 = 0;     // Interrupt-Anforderungsflag lschen  
}

#ifdef _EXT_INT0_
void ext0_dummy ( void ) interrupt 0
#endif
#ifdef _EXT_INT1_
void ext0_dummy ( void ) interrupt 2
#endif
{
  my_interrupt();      //  Aufruf der Benutzer-ISR
  TCON  &= ~(0x01 << IE_);  //  IE0 = 0;     // Interrupt-Anforderungsflag lschen
}

void ext_interrupt_enable ( void )
{
  IEN0  |= (0x01 << EX_);   //  EX0 = 1;
  IEN0  |= (0x01 << EA_);   //  EA  = 1;
}

void ext_interrupt_disable( void ){
  IEN0  &= ~(0x01 << EX_);  //  EX0 = 0;
  // !!globale Sperre nicht zurcksetzen, sonst Konflikt mit timer
}

//*******************************************************************
//** ab hier timer-Funktionen
//*******************************************************************
void timer1ms_init ( void (*ti) (void) )
{
  my_timer = ti;
  T2CON &= ~(0x01 << 5);   //  RCLK = 0;  // Timer2 im Autoreload-Betrieb
  T2CON &= ~(0x01 << 4);   //  TCLK = 0;
  T2CON &= ~(0x01 << 0);   //  CP_RL2 = 0;
  T2CON &= ~(0x01 << 1);   //  C_T2 = 0;  // fclk = 2MHz (bei 24MHz F_CPU)
  
  // berlauf (Interrupt) nach 1000 Clock-Cycles
  // ==> T_Preload = 65536 - (t * fclk) = 65536 - 1ms * 2 MHz
  //     T_Preload = 63536 = 0xF830
  
  #define T_PRELOAD   0x10000UL - 1 * F_CPU / 12 / 1000
  
  // Interrupt-Tick = 1ms einstellen
  RCAP2H = TH2 = (T_PRELOAD & 0xff00) >> 8;   //   0xF8
  RCAP2L = TL2 = (T_PRELOAD & 0x00ff);        //   0x30
}

void timer1ms_enable ( void )
{
  T2CON |= (0x01 << 2);    //  TR2 = 1;   // Timer2 starten
  IEN0  |= (0x01 << 5);    //  ET2 = 1;   // Timer2 Interrupt freigeben
  IEN0  |= (0x01 << 7);    //  EA  = 1;
}

void timer1ms_disable ( void )
{
  T2CON &= ~(0x01 << 2);   //  TR2 = 0;   // Timer2 stoppen
  IEN0  &= ~(0x01 << 5);   //  ET2 = 0;   // Timer2 Interrupt sperren
  // Globale Interruptfreigabe nicht sperren!! Konflikt mit ext_interrupt !!
}
// Die timer-Interrupt-Serviceroutine timer1ms_isr() muss
// selbst geschrieben werden!!
void timer2_dummy(void) interrupt 5
{
  my_timer();              //  aufrufen der Interrupt-Serviceroutine  
  T2CON &= ~(0x01 << 7);   //  TF2 = 0;   // Timer2 Interruptflag zurcksetzen.
}
