// Technische Richtlinie FA205 
// Bibliothek:       lcd.c
// Controller:       AT89C5131
// Version:          1.0
// erstellt am:      17.02.2015
// letzte nderung:  31.03.2016
// Autor:            Rahm

// Headerdateien inkludieren 
#include "lcd.h"

// Deklaration lokale Funktionen
uint8_t lcd_lookup (uint8_t ascii);
void    lcd_befehl (uint8_t befehl);
void    lcd_daten  (uint8_t daten);
void    lcd_waitwhilebusy (void);
void    init_wait  (uint16_t n);

// Wartezeit-Konstante fr Initialisierung und Busy-Abfrage berechnen
#define LCD_WAIT F_CPU/12000000

// Definition der Funktionen
// LCD lowlevel-Funktionen: lcd_waitwhilebusy(), lcd_befehl(), lcd_daten() und lcd_init()
// mssen fr jeden Controller/Board angepasst werden.
void lcd_waitwhilebusy(void)
{
  #ifdef LCD_BUSY
  uint8_t busy;
	
  LCD_PORT |=  (1<<RW);                 // R/W = 1  (Lesen)	
  do
  {
    LCD_PORT |= 0x0F;
    LCD_PORT |=  (1<<EN);               // EN = 1
    busy = LCD_PORT & (1<<LCD_BUSY);    // Read Busy Flag (und Address High Nibble (Pin!))
    LCD_PORT &= ~(1<<EN);               // EN = 0

    LCD_PORT |=  (1<<EN);               // EN = 1 Read Address low nibble (dummy)
    LCD_PORT &= ~(1<<EN);               // EN = 0
  } while (busy != 0);

  LCD_PORT &= ~(1<<RW);                 // R/W = 0  (Schreiben)

  #else  // Wenn kein Busy-Flag abgefragt wird!!
   init_wait(LCD_WAIT*80);
  #endif
}

/*****************************************************************************************
 * 	Gibt das Byte 'befehl' als Befehl ans LCD-Display im 4Bit-Modus        
 *****************************************************************************************/
void lcd_befehl (uint8_t befehl)
{	
  uint8_t temp;
		
  lcd_waitwhilebusy();                  // Warten bis LCD bereit!
		
  temp = befehl;
  temp >>= 4;										
  temp &= 0x0F;
  LCD_PORT = temp;                      // High-Nibble auf Datenleitungen legen
  LCD_PORT |= (1<<EN);                  // EN = 1
  LCD_PORT &= ~(1<<EN);                 // EN = 0
																
  temp = befehl;                        // Low-Nibbel
  temp &= 0x0F;
  LCD_PORT = temp;                      // Low-Nibble auf Datenleitungen legen
  LCD_PORT |= (1<<EN);                  // TEN = 1
  LCD_PORT &= ~(1<<EN);                 // EN = 0
}

/*****************************************************************************************
 * 	Gibt das Byte 'daten' ans LCD-Display im 4Bit-Modus aus.     
 *****************************************************************************************/
void lcd_daten(unsigned char daten)
{
  uint8_t temp;
	
  lcd_waitwhilebusy();                  // Warten bis LCD bereit!
	
  temp = daten ;
  temp >>= 4;		
	
  temp &= 0x0F;
  LCD_PORT = temp;                      // High-Nibble auf Datenleitungen legen
  LCD_PORT |= (1<<RS);                  // Data-Register On
  LCD_PORT |= (1<<EN);                  // EN = 1
  LCD_PORT &= ~(1<<EN);                 // EN = 0

  temp = daten;                         // Low-Nibbel		
  temp &= 0x0F;
  LCD_PORT = temp;                      // Low-Nibble auf Datenleitungen legen
  LCD_PORT |= (1<<RS);                  // Data-Register On
  LCD_PORT |= (1<<EN);                  // EN = 1
  LCD_PORT &= ~(1<<EN);                 // EN = 0
  LCD_PORT &= ~(1<<RS);                 // Data-Register Off
}

/******************************************************************************************
 * Initialisierung des LCD- Displays fr den 4-Bit-Modus                                         
 ******************************************************************************************/
void lcd_init (void)	
{ 
  LCD_PORT = 0x00;
  LCD_PORT = 0x03;							
  LCD_PORT |= (1<<EN);                  // 3 x aufwecken!  EN = 1	
  LCD_PORT &= ~(1<<EN);                 //                 EN = 0
	
  init_wait(LCD_WAIT*100);	 													
  LCD_PORT |= (1<<EN);
  LCD_PORT &= ~(1<<EN);
		
  init_wait(LCD_WAIT*2); 
  LCD_PORT |= (1<<EN);
  LCD_PORT &= ~(1<<EN);

  init_wait(LCD_WAIT*2);
  LCD_PORT = 0x02;
  LCD_PORT |= (1<<EN);
  LCD_PORT &= ~(1<<EN);
														
  init_wait(LCD_WAIT*2);
  lcd_befehl (0x28);                     // Function set 4 bits  													
  lcd_befehl (CURSOR);                   // Display AN, Cursor AUS													
  lcd_befehl (DISPLAY);                  // Not Shifted Display, Increment				
}

/******************************************************************************************
 * Display lschen	                                                                                                      
 ******************************************************************************************/
void lcd_clear (void)   {lcd_befehl (0x01);}
	
/*****************************************************************************************
 * LCD-Cursor auf eine position setzen: Zeile 1..4, Spalte 1..16 (Displayabhngig)               
 ******************************************************************************************/
void lcd_setcursor (uint8_t row, uint8_t column)
{ 	
  uint8_t position;
  switch(row)
  {
    case 1:  position = ZEILE1; break;
    case 2:  position = ZEILE2; break;
    case 3:  position = ZEILE3; break;
    case 4:  position = ZEILE4; break;	
    default: position = ZEILE1; break;		
  }
	
  position += (column-1);
		
  lcd_befehl ( position | 0x80 );         // 0x80 = Kennung fr DD RAM address set
}

/*****************************************************************************************
 *  Ausgabe eines Zeichens an das LCD-Display                                                          
 *  Entweder Busy-Flag des LCD abfragen oder Zeitverzgerung verwenden         
 ******************************************************************************************/
void lcd_char (uint8_t value)
{
  value = lcd_lookup(value);            // Umlaute und Sonderzeichen ersetzen!
  lcd_daten(value);
}

/*****************************************************************************************
 * \0-terminierten Text an das LCD-Display ausgeben. 
 ******************************************************************************************/
void lcd_print (uint8_t *text)
{				
  while (*text != '\0')                   // Text													
    lcd_char (*text++);                   // zeichenweise ausgeben 	
}

/*****************************************************************************************
 * Eine 3 stellige Zahl auf die aktuelle Cursorposition ausgeben.
 ******************************************************************************************/
void lcd_byte(uint8_t value)
{
  uint8_t buffer[3];
  uint8_t n = 0;	
	
  do
  {
    buffer[n++] = value%10 + '0';
  } while ((value /= 10) > 0);
				
  while (n<3)                             // Rest von buffer mit blank fllen
  {
    buffer[n++] = ' ';					
  }

  while (n > 0)                           // Ausgabe auf das Display (umgekehrt)
  {
    n--;
    lcd_char(buffer[n]);
  }
}

/*****************************************************************************************
 * Gibt value als 5 stelligen Dez-Wert aufs Display. Fhrende Nullen werden zu blank.
 ******************************************************************************************/
void lcd_int(uint16_t value)
{	
  uint8_t buffer[5];
  uint8_t n = 0;	
	
  do
  {
    buffer[n++] = value%10 + '0';
  } while ((value /= 10) > 0);
		
  while (n<5)                             // Rest von buffer mit blank fllen
  {
    buffer[n++] = ' ';					
  }

  while (n > 0)                           // Ausgabe auf das Display (umgekehrt)
  {
    n--;
    lcd_char(buffer[n]);
  }
}

/******************************************************************************************
 * Ersetzt ein ASCII-Umlaut durch den entsprechenden Display-Code.
 *******************************************************************************************/
uint8_t lcd_lookup(uint8_t ascii)
{
  switch (ascii)
  {
    case 0xb0: return 0xdf; // ''
    case '':  return 0xe1;
    case '':  return 0xef;
    case '':  return 0xf5;
    case '':  return 0xe1;
    case '':  return 0xef;
    case '':  return 0xf5;
    case '':  return 0xe2;
    case '':  return 0xe4;
    case '\\': return 0xa4;
    case '':  return 0xd3;
  }
  return ascii;
}
// Warteschleife fr Initialisierung, wenn Busy-Flag noch nicht gltig ist.
void init_wait(uint16_t n) {while(n--);}

/////////////////////////////////////////////////////////////////////////////////
// Ab hier sind ergnzende Funktionen definiert, die nicht Teil der technischen
// Richtlinie FA205 sind.
/////////////////////////////////////////////////////////////////////////////////
#ifdef LCD_EXTENSION
/***********************************************************************************
 * Pixelweise Definition eigener LCD-Zeichen, Ablegen von max 7 Zeichen im CG-RAM des LCD
 ***********************************************************************************/
void lcd_defchar (uint8_t *pixtab, uint8_t charNr)
{
  uint8_t z;
	
  if (charNr<8)                            // Adr 0 nicht innerhalb eines Textstrings nutzbar
  lcd_befehl (((charNr*8) & 0x7F) | 0x40);	
  // Adresse der ersten Pixelzeile = 8 x Adresse des fertigen Zeichens im DD Ram
  // Zugriff auf Adresse im CG-RAM (Character Generator) mit  Bit7 = 0, Bit6 = 1
  for (z=0;z<8;z++)	
    lcd_char (*(pixtab+z));                // 8 Pixelzeilen ins CR-Ram
  lcd_setcursor (1,1);                     // wieder auf DD-Ram-Zugriff umschalten		
}
/***************************************************************************
 * Lschen der Zeilen 1 bis 4 des Displays
 * Es wird ein Leerstring (16 Byte) auf die Zeile geschrieben
 ***************************************************************************/
void lcd_clearline(uint8_t lineNr)
{
  uint8_t n;

  lcd_setcursor(lineNr,0);
  for (n=0; n < LCD_LEN; n++)  lcd_char(' ');
}
#endif